package com.jpsoft.picc.interceptor;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.security.Keys;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.security.Key;
import java.util.Base64;

@Component
public class LoginInterceptor implements HandlerInterceptor {
    @Value("${jwt.secret}")
    private String jwtSecret;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if (request.getMethod().equals("OPTIONS")){
            return true;
        }

        String token = request.getHeader("Authorization");

        if (StringUtils.isEmpty(token)){
            token = (String)request.getSession().getAttribute("token");
        }

        if (StringUtils.isEmpty(token)) {
            token = request.getParameter("token");
        }

        boolean result = false;

        try {
            String prefix = "Bearer ";

            if (token==null || token.length() < prefix.length()){
                throw new Exception("未传递令牌或未带前缀Bearer!");
            }

            token = token.substring(prefix.length());

            byte[] privateKey = Base64.getDecoder().decode(jwtSecret);

            Key key = Keys.hmacShaKeyFor(privateKey);

            Claims claims = Jwts.parser()
                            .setSigningKey(key)
                            .parseClaimsJws(token)
                            .getBody();

            String userId = claims.getSubject();

            request.setAttribute("subject",userId);

            result = true;
        }
        catch(Exception ex){
            MessageResult<String> msgResult = new MessageResult<>();

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
            msgResult.setCode(415);

            ObjectMapper mapper = new ObjectMapper();
            String json = mapper.writeValueAsString(msgResult);

            response.setContentType("application/json");
            response.setCharacterEncoding("UTF-8");
            response.setHeader("access-control-allow-origin","*");
            response.getWriter().print(json);

            result = false;
        }

        return result;
    }
}
