package com.jpsoft.picc.modules.base.controller;

import cn.hutool.core.io.FileUtil;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Element;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.CompanyMember;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinitionLimit;
import com.jpsoft.picc.modules.base.service.CompanyMemberService;
import com.jpsoft.picc.modules.base.service.CompanyService;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionLimitService;
import com.jpsoft.picc.modules.business.entity.InsuranceApplication;
import com.jpsoft.picc.modules.business.entity.InsurancePolicy;
import com.jpsoft.picc.modules.business.entity.InsurancePolicyMember;
import com.jpsoft.picc.modules.business.service.ApplicationPolicyService;
import com.jpsoft.picc.modules.business.service.InsuranceApplicationService;
import com.jpsoft.picc.modules.business.service.InsurancePolicyMemberService;
import com.jpsoft.picc.modules.business.service.InsurancePolicyService;
import com.jpsoft.picc.modules.common.config.OSSConfig;
import com.jpsoft.picc.modules.common.config.PdfConfig;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.utils.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.util.StringUtil;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(description="附件管理")
@Controller
@Slf4j
public class AttachmentController {
    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private PdfConfig pdfConfig;

    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private InsurancePolicyService insurancePolicyService;

    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;

    @Autowired
    private InsurancePolicyMemberService insurancePolicyMemberService;

    @Autowired
    private ApplicationPolicyService applicationPolicyService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private CompanyMemberService companyMemberService;

    @Value("${insurance.stampFileUrl}")
    private String defaultStampFileUrl;

    @PostMapping("/base/attachment/upload")
    @ResponseBody
    @ApiOperation(value="附件上传")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "fileName",value = "文件名", required = false, paramType = "form"),
            @ApiImplicitParam(name = "subFolder",value = "子目录名(如/test)", required = false, paramType = "form"),
            @ApiImplicitParam(name = "uploadFile",value = "上传文件", required = true,paramType="form", dataType = "__file")
    })
    public MessageResult<String> upload(String fileName, String subFolder, MultipartFile uploadFile){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            if (StringUtils.isEmpty(fileName)){
                fileName = uploadFile.getOriginalFilename();
            }

            String retFileUrl = OSSUtil.upload(ossConfig,subFolder,fileName,uploadFile.getInputStream());

            messageResult.setResult(true);
            messageResult.setData(retFileUrl);
            messageResult.setCode(200);
        } catch (Exception e) {
            log.error(e.getMessage(),e);

            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }

    public boolean transferAlpha2File(String imgSrc, String imgTarget) {
        File file = new File(imgSrc);
        InputStream is = null;
        boolean result = false;

        try {
            is = new FileInputStream(file);
            // 如果是MultipartFile类型，那么自身也有转换成流的方法：is = file.getInputStream();
            BufferedImage bi = ImageIO.read(is);
            Image image = (Image) bi;
            ImageIcon imageIcon = new ImageIcon(image);
            BufferedImage bufferedImage = new BufferedImage(imageIcon.getIconWidth(), imageIcon.getIconHeight(),
                    BufferedImage.TYPE_4BYTE_ABGR);
            Graphics2D g2D = (Graphics2D) bufferedImage.getGraphics();
            g2D.drawImage(imageIcon.getImage(), 0, 0, imageIcon.getImageObserver());
            int alpha = 0;
            for (int j1 = bufferedImage.getMinY(); j1 < bufferedImage.getHeight(); j1++) {
                for (int j2 = bufferedImage.getMinX(); j2 < bufferedImage.getWidth(); j2++) {
                    int rgb = bufferedImage.getRGB(j2, j1);
                    int R = (rgb & 0xff0000) >> 16;
                    int G = (rgb & 0xff00) >> 8;
                    int B = (rgb & 0xff);
                    if (((255 - R) < 30) && ((255 - G) < 30) && ((255 - B) < 30)) {
                        rgb = ((alpha + 1) << 24) | (rgb & 0x00ffffff);
                    }
                    bufferedImage.setRGB(j2, j1, rgb);
                }
            }
            g2D.drawImage(bufferedImage, 0, 0, imageIcon.getImageObserver());
            result = ImageIO.write(bufferedImage, "png", new File(imgTarget));// 直接输出文件
        } catch (Exception e) {
            e.printStackTrace();
            result = false;
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                }
            }
        }
        return result;
    }

    @ApiOperation(value="pdf文档生成与下载")
    @RequestMapping(value = "/pub/attachment/downloadPolicyPdf",method = RequestMethod.GET)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = false,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query")
    })
    public ResponseEntity downloadPolicyPdf(String applicationId, String policyId){
        String logoUrl = pdfConfig.getLogoUrl();
        ResponseEntity entity;

        try {
            InsuranceApplication insuranceApplication = null;

            if (StringUtils.isEmpty(applicationId)){
                List<InsuranceApplication> applicationList = applicationPolicyService.findApplicationByPolicyId(policyId);

                if (applicationList.size()>0){
                    insuranceApplication = applicationList.get(0);
                }
            }
            else {
                insuranceApplication = insuranceApplicationService.get(applicationId);
            }

            Company company = companyService.get(insuranceApplication.getCompanyId());


            List<InsuranceDefinitionLimit> insuranceDefinitionLimitList = insuranceDefinitionLimitService.findByDefinitionId(insuranceApplication.getDefinitionId());

            List<Map<String, Object>> mapList = new ArrayList<Map<String, Object>>();

            Map<String,Object> varsMap = new HashMap<>();
            varsMap.put("num",insuranceApplication.getInsuredNumber());

            for(InsuranceDefinitionLimit  insuranceDefinitionLimit:insuranceDefinitionLimitList){
                Map<String,Object> map1 = new HashMap<>();
                map1.put("id",insuranceDefinitionLimit.getId());
                map1.put("name",insuranceDefinitionLimit.getName());

                //限额这里可能是公式，需要带入人数
                String limit = VelocityHelper.format(insuranceDefinitionLimit.getLimit(),varsMap);
                map1.put("limit",limit);

                String unit = VelocityHelper.format(insuranceDefinitionLimit.getUnit(),varsMap);
                map1.put("unit",unit);

                mapList.add(map1);
            }

            /*for (InsuranceDefinitionLimit insuranceDefinitionLimit : insuranceDefinitionLimitList) {
                Map<String, Object> map1 = new HashMap<String, Object>();
                map1.put("id", insuranceDefinitionLimit.getId());
                map1.put("name", insuranceDefinitionLimit.getName());
                map1.put("limit", insuranceDefinitionLimit.getLimit());
                map1.put("unit", insuranceDefinitionLimit.getUnit());
                mapList.add(map1);
            }*/

            List<InsurancePolicyMember> list = insurancePolicyMemberService.findByPolicyId(policyId);

            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);

            //设置编码  为了解决中文名称乱码问题
            String fileName = "投保单电子版.pdf";
            String downloadFileName = new String(fileName.getBytes("UTF-8"), "iso-8859-1");

            //将编码加到http头信息中
            headers.setContentDispositionFormData("attachment", downloadFileName);

            String folderPath = pdfConfig.getOutputPath();

            File folder = new File(folderPath);

            if (!folder.exists()){
                folder.mkdirs();
            }

            String filePath = folderPath + File.separator + DateTime.now().toString("yyyyMMddHHmmssSSS") + ".pdf";

            log.warn("生成PDF文档：" + filePath);

            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);

            ItextPDFUtil.createPdf(insuranceApplication,insurancePolicy, mapList, list, logoUrl,filePath);

            String watermarkFilePath = folderPath + File.separator + DateTime.now().toString("yyyyMMddHHmmssSSS") + "-watermark.pdf";

            //pdf文档添加图片水印
            if (StringUtils.isNotEmpty(company.getCompanyStampFile())) {
                String ext = company.getCompanyStampFile().substring(company.getCompanyStampFile().lastIndexOf("."));
                String watermarkLogoPath =  folderPath + File.separator + DateTime.now().toString("yyyyMMddHHmmssSSS") + ext;
                String watermarkLogoPath2 =  folderPath + File.separator + DateTime.now().toString("yyyyMMddHHmmssSSS") + "-t" + ext;

                //下载水印图片
                boolean success = OSSUtil.download(company.getCompanyStampFile(),watermarkLogoPath);

                if(success) {
                    try {
                        //水印背景变透明
                        transferAlpha2File(watermarkLogoPath,watermarkLogoPath2);
                        Watermark.imageWatermark(filePath, watermarkFilePath, watermarkLogoPath2, defaultStampFileUrl);
                        filePath = watermarkFilePath;
                    }
                    catch (Exception e){
                        log.error(e.getMessage(),e);
                    }
                }
            }

            byte[] buffer = FileUtil.readBytes(filePath);

            entity = new ResponseEntity<byte[]>(buffer, headers, HttpStatus.OK);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
            entity = new ResponseEntity<String>(ex.getMessage(),HttpStatus.OK);
        }

        return entity;
    }

    @ApiOperation(value="被保险人列表")
    @RequestMapping(value = "/pub/attachment/downloadPolicyMemberXls",method = RequestMethod.GET)
    @ApiImplicitParams({
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query")
    })
    public ResponseEntity downloadPolicyMemberXls(String policyId){
        ResponseEntity entity;

        try {
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);

            //设置编码  为了解决中文名称乱码问题
            String fileName = "被保险人列表.xls";
            String downloadFileName = new String(fileName.getBytes("UTF-8"), "iso-8859-1");

            //将编码加到http头信息中
            headers.setContentDispositionFormData("attachment", downloadFileName);

            Workbook workbook = new HSSFWorkbook();
            Sheet sheet = workbook.createSheet();

            //表头
            Row rowTitle = sheet.createRow(0);

            Cell cellTitle1 = rowTitle.createCell(0);
            cellTitle1.setCellValue("序号");

            Cell cellTitle2 = rowTitle.createCell(1);
            cellTitle2.setCellValue("姓名");

            Cell cellTitle3 = rowTitle.createCell(2);
            cellTitle3.setCellValue("证件类型");

            Cell cellTitle4 = rowTitle.createCell(3);
            cellTitle4.setCellValue("证件号码");

            Cell cellTitle5 = rowTitle.createCell(4);
            cellTitle5.setCellValue("性别");

            Cell cellTitle6 = rowTitle.createCell(5);
            cellTitle6.setCellValue("年龄");

            Cell cellTitle7 = rowTitle.createCell(6);
            cellTitle7.setCellValue("岗位/工种");

            Cell cellTitle8 = rowTitle.createCell(7);
            cellTitle8.setCellValue("健康情况");

            List<InsurancePolicyMember> memberList = insurancePolicyMemberService.findByPolicyId(policyId);


            for (int i =0;i<memberList.size();i++){
                InsurancePolicyMember member = memberList.get(i);

                Row row = sheet.createRow(i+1);
                row.createCell(0).setCellValue(String.valueOf(i+1));
                row.createCell(1).setCellValue(member.getCompanyMember().getName());
                row.createCell(2).setCellValue(member.getCompanyMember().getCardTypeN());
                row.createCell(3).setCellValue(member.getCompanyMember().getCardNo());
                row.createCell(4).setCellValue(member.getCompanyMember().getSex() ? "男" : "女");
                row.createCell(5).setCellValue(member.getCompanyMember().getAge());
                row.createCell(6).setCellValue(member.getCompanyMember().getJobName());
                row.createCell(7).setCellValue(member.getCompanyMember().getHealthStatus());
             }

            ByteArrayOutputStream output = new ByteArrayOutputStream();
            workbook.write(output);

            entity = new ResponseEntity<>(output.toByteArray(), headers, HttpStatus.OK);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
            entity = new ResponseEntity<String>(ex.getMessage(),HttpStatus.OK);
        }

        return entity;
    }

    @ApiOperation(value="身份证附件打包下载")
    @RequestMapping(value = "/pub/attachment/downloadIdCardZip",method = RequestMethod.GET)
    @ApiImplicitParams({
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query")
    })
    public void downloadIdCardZip(String policyId, HttpServletResponse response){
        try {
            response.setCharacterEncoding("utf-8");
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);

            //设置编码  为了解决中文名称乱码问题
            String fileName = "身份证";
            String zipFileName = new String(fileName.getBytes("UTF-8"), "iso-8859-1");
            response.setHeader("Content-Disposition", "attachment;fileName=" + zipFileName + ".zip");

            List<InsurancePolicyMember> policyMemberList = insurancePolicyMemberService.findByPolicyId(policyId);

            List<Map<String,String>> fileList = new ArrayList<>();

            for (InsurancePolicyMember policyMember : policyMemberList) {
                CompanyMember member = companyMemberService.get(policyMember.getMemberId());

                String urls = member.getCardFiles();

                if (StringUtils.isNotEmpty(urls)){
                    String[] arr = urls.split("\\|");

                    for(int i=0;i<arr.length;i++) {
                        String url = arr[i];

                        String ext = url.substring(url.lastIndexOf("."));

                        Map<String,String> map = new HashMap<>();
                        map.put("fileUrl",url + "?x-oss-process=image/resize,l_1024,limit_1");
                        map.put("fileName", member.getCardNo() + "-" + (i+1) + ext);

                        fileList.add(map);
                    }
                }
            }

            OutputStream output = response.getOutputStream();

            OSSUtil.batchDownload(fileList,output);

            output.flush();
            output.close();
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
        }
    }


    @ApiOperation(value="打包下载申请资料")
    @RequestMapping(value = "/pub/attachment/downloadPolicyZip",method = RequestMethod.GET)
    @ApiImplicitParams({
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query")
    })
    public void downloadPolicyZip(String policyId, HttpServletRequest request, HttpServletResponse response){
        try {
            String path = request.getContextPath();
            String basePath = request.getScheme()+"://"+request.getServerName()+":"+request.getServerPort()+path+"/";

            response.setCharacterEncoding("utf-8");
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);

            //设置编码  为了解决中文名称乱码问题
            String fileName = "投保申请资料";
            String zipFileName = new String(fileName.getBytes("UTF-8"), "iso-8859-1");
            response.setHeader("Content-Disposition", "attachment;fileName=" + zipFileName + ".zip");

            //打包文件
            List<Map<String,String>> fileList = new ArrayList<>();

            //营业执照
            InsuranceApplication insuranceApplication = null;

            List<InsuranceApplication> applicationList = applicationPolicyService.findApplicationByPolicyId(policyId);

            if (applicationList.size()>0){
                insuranceApplication = applicationList.get(0);
            }

            if (insuranceApplication!=null){
                Company company = companyService.get(insuranceApplication.getCompanyId());
                String url = company.getUsccFiles();

                if(StringUtils.isNotEmpty(url)) {
                    Map<String, String> fileMap = new HashMap<>();
                    fileMap.put("fileUrl", url);
                    fileMap.put("fileName", "营业执照" + url.substring(url.lastIndexOf(".")));

                    fileList.add(fileMap);
                }
            }

            //电子投保文件
            if(insuranceApplication!=null){
                Map<String,String> fileMap = new HashMap<>();
                fileMap.put("fileUrl",basePath + "pub/attachment/downloadPolicyPdf?applicationId="
                        + insuranceApplication.getId()
                        + "&policyId=" + policyId);

                fileMap.put("fileName", "投保文件电子版.pdf");

                fileList.add(fileMap);
            }

            //人员名单.xls
            if(StringUtils.isNotEmpty(policyId)){
                Map<String,String> fileMap = new HashMap<>();
                fileMap.put("fileUrl",basePath + "pub/attachment/downloadPolicyMemberXls?policyId=" + policyId);
                fileMap.put("fileName", "投保人列表.xls");

                fileList.add(fileMap);
            }

            //身份证
            List<InsurancePolicyMember> policyMemberList = insurancePolicyMemberService.findByPolicyId(policyId);

            for (InsurancePolicyMember policyMember : policyMemberList) {
                CompanyMember member = companyMemberService.get(policyMember.getMemberId());

                String urls = member.getCardFiles();

                if (StringUtils.isNotEmpty(urls)){
                    String[] arr = urls.split("\\|");

                    for(int i=0;i<arr.length;i++) {
                        String url = arr[i];

                        String ext = url.substring(url.lastIndexOf("."));

                        Map<String,String> fileMap = new HashMap<>();
                        fileMap.put("fileUrl",url + "?x-oss-process=image/resize,l_1024,limit_1");
                        fileMap.put("filePath", "身份证/");
                        fileMap.put("fileName", member.getCardNo() + "-" + (i+1) + ext);

                        fileList.add(fileMap);
                    }
                }
            }

            OutputStream output = response.getOutputStream();

            OSSUtil.batchDownload(fileList,output);

            output.flush();
            output.close();
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
        }
    }
}