package com.jpsoft.picc.modules.auth.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.CompanyUser;
import com.jpsoft.picc.modules.base.service.CompanyService;
import com.jpsoft.picc.modules.base.service.CompanyUserService;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.entity.CompanyInvoice;
import com.jpsoft.picc.modules.base.service.CompanyInvoiceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/auth/companyInvoice")
@Api(description = "companyInvoice")
public class CompanyInvoiceController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private CompanyService companyService;
    @Autowired
    private CompanyUserService companyUserService;
    @Autowired
    private CompanyInvoiceService companyInvoiceService;

    @GetMapping(value="detail")
    @ApiOperation(value = "获取发票信息")
    public MessageResult<CompanyInvoice> detail(HttpServletRequest request){
        MessageResult<CompanyInvoice> messageResult = new MessageResult<>();

        try {
            //todo
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            CompanyInvoice companyInvoice = companyInvoiceService.getByCompanyId(companyUser.getCompanyId());

            if(companyInvoice == null) {
                companyInvoice = new CompanyInvoice();
            }

            messageResult.setResult(true);
            messageResult.setData(companyInvoice);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping(value="save")
    @ApiOperation(value = "保存发票信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type",value = "发票类型", paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "title",value = "发票抬头", paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "identifier",value = "税号", paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "bank",value = "开户银行", paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "companyAccount",value = "企业账号", paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "bankAccount",value = "联行号", paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "address",value = "企业地址", paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "telephone",value = "联系电话", paramType = "form",dataType = "String")
    })
    public MessageResult<CompanyInvoice> save(
                                      @RequestParam(value="type",defaultValue="") String type,
                                      @RequestParam(value="title",defaultValue="null") String title,
                                      @RequestParam(value="identifier",defaultValue="null") String identifier,
                                      @RequestParam(value="bank",defaultValue="null") String bank,
                                      @RequestParam(value="companyAccount",defaultValue="null") String companyAccount,
                                      @RequestParam(value="bankAccount",defaultValue="null") String bankAccount,
                                      @RequestParam(value="address",defaultValue="null") String address,
                                      @RequestParam(value="telephone",defaultValue="null") String telephone,
                                      HttpServletRequest request){
        MessageResult<CompanyInvoice> messageResult = new MessageResult<>();

        try {
            //todo
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());

            CompanyInvoice companyInvoice = companyInvoiceService.getByCompanyId(companyUser.getCompanyId());

            if(companyInvoice != null){
                companyInvoice.setUpdateBy(principal.getName());
                companyInvoice.setUpdateTime(new Date());

                companyInvoice.setType(type);
                companyInvoice.setTitle(title);
                companyInvoice.setIdentifier(identifier);
                companyInvoice.setBank(bank);
                companyInvoice.setCompanyAccount(companyAccount);
                companyInvoice.setBankAccount(bankAccount);
                companyInvoice.setAddress(address);
                companyInvoice.setTelephone(telephone);

                companyInvoiceService.update(companyInvoice);
            }else{
                companyInvoice = new CompanyInvoice();

                companyInvoice.setId(UUID.randomUUID().toString());
                companyInvoice.setCompanyId(companyUser.getCompanyId());
                companyInvoice.setDelFlag(false);
                companyInvoice.setCreateBy(principal.getName());
                companyInvoice.setCreateTime(new Date());

                companyInvoice.setType(type);
                companyInvoice.setTitle(title);
                companyInvoice.setIdentifier(identifier);
                companyInvoice.setBank(bank);
                companyInvoice.setCompanyAccount(companyAccount);
                companyInvoice.setBankAccount(bankAccount);
                companyInvoice.setAddress(address);
                companyInvoice.setTelephone(telephone);

                companyInvoiceService.insert(companyInvoice);
            }

            messageResult.setResult(true);
            messageResult.setData(companyInvoice);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
