package com.jpsoft.picc.modules.auth.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.auth.dto.AcceptInsuranceDTO;
import com.jpsoft.picc.modules.auth.dto.InsuranceApplicationDTO;
import com.jpsoft.picc.modules.auth.dto.InsuranceJobsDTO;
import com.jpsoft.picc.modules.base.entity.*;
import com.jpsoft.picc.modules.base.service.*;
import com.jpsoft.picc.modules.business.entity.*;
import com.jpsoft.picc.modules.business.service.*;
import com.jpsoft.picc.config.JpCloudConfig;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.utils.SignUtil;
import com.jpsoft.picc.modules.common.utils.VelocityHelper;
import com.jpsoft.picc.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hpsf.Decimal;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.joda.time.Interval;
import org.joda.time.Months;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.security.Policy;
import java.text.DecimalFormat;
import java.util.*;

@Slf4j
@Api(description="投保信息")
@RestController
@RequestMapping("/auth/insuranceApplication")
public class InsuranceApplicationController {
    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private InsurancePolicyService insurancePolicyService;

    @Autowired
    private InsuranceDefinitionService insuranceDefinitionService;

    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;

    @Autowired
    private InsuranceJobsService insuranceJobsService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private InsurancePolicyMemberService insurancePolicyMemberService;

    @Autowired
    private JobsService jobsService;

    @Autowired
    private ApplicationPolicyService applicationPolicyService;

    @Autowired
    private JpCloudConfig jpCloudConfig;

    @Autowired
    private TransactionRecordService transactionRecordService;

    @Autowired
    private InsurancePolicyRecordService insurancePolicyRecordService;

    @Autowired
    private CompanyUserService companyUserService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private TemplateMessageService templateMessageService;

    @Value("${insurance.insuredName}")
    private String defaultInsureName;

    @Value("${insurance.insuredTel}")
    private String defaultInsureTel;

    @ApiOperation(value="投保单列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(name = "status",value = "状态:草稿10/已退回15/待初审20/待复审30/待缴费40/待制单50/待出单60/已出单70/已过期100", required = true, paramType = "form",dataType = "String"),
        @ApiImplicitParam(name = "definitionId",value = "险种编号", required = false, paramType = "form",dataType = "String"),
        @ApiImplicitParam(name = "startTime",value = "起始时间", required = false, paramType = "form",dataType = "String"),
        @ApiImplicitParam(name = "endTime",value = "结束时间", required = false, paramType = "form",dataType = "String"),
    })
    public MessageResult<Map> pageList(
            String status,String definitionId,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date startTime,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date endTime,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            Company company = null;

            if (companyUser!=null && StringUtils.isNotEmpty(companyUser.getCompanyId())) {
                company = companyService.get(companyUser.getCompanyId());
            }

            if (company==null){
                throw new Exception("请先填写企业信息!");
            }

            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            if (StringUtils.isNotEmpty(company.getId())) {
                searchParams.put("companyId", company.getId());
            }

            if (StringUtils.isNotEmpty(definitionId)) {
                searchParams.put("definitionId", definitionId);
            }

            if (StringUtils.isNotEmpty(status)) {
                searchParams.put("status", status);
            }

            if (startTime != null) {
                searchParams.put("startTime", startTime);
            }

            if (endTime != null) {
                searchParams.put("endTime", endTime);
            }

            Page<InsuranceApplication> page = insuranceApplicationService.pageSearch(searchParams, pageIndex, pageSize, sortList);

            msgResult.setData(PojoUtils.pageWrapper(page));
            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="第一步，获取投保信息，注意将applicationId,policyId,companyId保存到前台，便于关联")
    @RequestMapping(value = "insureDetail",method = RequestMethod.GET)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "applicationId",value = "投保编号", required = false, paramType = "query",dataType = "String")
    })
    public MessageResult<InsuranceApplicationDTO> insureDetail(@RequestParam(value="applicationId",defaultValue="") String applicationId,
                                                          HttpServletRequest request){
        MessageResult<InsuranceApplicationDTO> msgResult = new MessageResult<>();

        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
        Company company = companyService.get(companyUser.getCompanyId());

        if(company == null){
            msgResult.setResult(false);
            msgResult.setMessage("请先完善企业信息！");

            return msgResult;
        }

        InsuranceApplicationDTO insuranceApplicationDTO = new InsuranceApplicationDTO();

        try {

            InsuranceApplication insuranceApplication = null;

            if (StringUtils.isNotEmpty(applicationId)) {
                insuranceApplication = insuranceApplicationService.get(applicationId);

                if(insuranceApplication==null){
                    throw new Exception("投保单不存在!");
                }
            }

            if (insuranceApplication != null) {
                PojoUtils.map(insuranceApplication, insuranceApplicationDTO);
                insuranceApplicationDTO.setApplicationId(insuranceApplication.getId());

                //营业性质
//                if (StringUtils.isNotEmpty(insuranceApplication.getBusinessNature())){
//                    String businessNatureName = dataDictionaryService.findNameByCatalogNameAndValue("营业性质",insuranceApplication.getBusinessNature());
//                    insuranceApplicationDTO.setBusinessNatureN(businessNatureName);
//                }

                DateTime endTime = new DateTime(insuranceApplication.getEndTime());
                DateTime startTime = new DateTime(insuranceApplication.getStartTime());

                insuranceApplicationDTO.setMonthNumber(Months.monthsBetween(startTime, endTime.plusDays(1)).getMonths());
            } else {
                //先生成主键，便于参保人员关联
                insuranceApplicationDTO.setApplicationId(UUID.randomUUID().toString());

                insuranceApplicationDTO.setCompanyId(company.getId());
//                insuranceApplicationDTO.setInsuredName(company.getName());
//                insuranceApplicationDTO.setInsuredTel(company.getLegalPhone());
                //平台上涉及投保人名称的统一默认为：湖北远鹏众创空间科技有限公司；联系电话：13907216102
                insuranceApplicationDTO.setInsuredName(defaultInsureName);
                insuranceApplicationDTO.setInsuredTel(defaultInsureTel);

                insuranceApplicationDTO.setCompanyName(company.getName());
                insuranceApplicationDTO.setBusinessNature(company.getBusinessNature());
                insuranceApplicationDTO.setUsccCode(company.getUsccCode());
                insuranceApplicationDTO.setBusinessScope(company.getBusinessScope());
                insuranceApplicationDTO.setIndustryType(company.getIndustryType());
                insuranceApplicationDTO.setEmployeesNumber(company.getEmployeesNumber());
                insuranceApplicationDTO.setInsuredNumber(0);
                insuranceApplicationDTO.setTel(company.getTel());
                insuranceApplicationDTO.setPostal(company.getPostal());
                insuranceApplicationDTO.setCompanyAddress(company.getAddress());
                insuranceApplicationDTO.setLossInRecentYears(company.getRemark());
            }

            //读取每月投保单编号
            List<ApplicationPolicy> relatePolicyList = applicationPolicyService.findByApplicationId(applicationId);

            if (relatePolicyList.size()==0){
                //生成首月投保编号
                insuranceApplicationDTO.setPolicyId(UUID.randomUUID().toString());
            }
            else{
                //首月投保
                insuranceApplicationDTO.setPolicyId(relatePolicyList.get(0).getPolicyId());
            }

            msgResult.setData(insuranceApplicationDTO);

            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="第三步，获取承保信息")
    @RequestMapping(value = "acceptDetail",method = RequestMethod.GET)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "applicationId",value = "投保单ID", required = true,paramType = "query",dataType = "String"),
            @ApiImplicitParam(name = "policyId",value = "每月投保单ID", required = true,paramType = "query",dataType = "String"),
    })
    public MessageResult<AcceptInsuranceDTO> acceptDetail(@RequestParam(value="applicationId",defaultValue="") String applicationId,
                                                          @RequestParam(value="policyId",defaultValue="") String policyId){
        MessageResult<AcceptInsuranceDTO> msgResult = new MessageResult<>();
        AcceptInsuranceDTO acceptInsuranceDTO = new AcceptInsuranceDTO();

        try {
            //投保单
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);

            //险种
            InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(insuranceApplication.getDefinitionId());

            //险种名称
            acceptInsuranceDTO.setInsuranceDefName(insuranceDefinition.getName());

            //关联岗位
            List<InsuranceJobs> InsuranceJobsList = insuranceJobsService.findByDefinitionId(insuranceApplication.getDefinitionId());

            DateTime endTime = new DateTime(insuranceApplication.getEndTime());
            acceptInsuranceDTO.setEndTime(insuranceApplication.getEndTime());

            DateTime startTime = new DateTime(insuranceApplication.getStartTime());
            acceptInsuranceDTO.setStartTime(insuranceApplication.getStartTime());

            //投保总月份
            int months = Months.monthsBetween(startTime, endTime.plusDays(1)).getMonths();

            BigDecimal totalAmount = new BigDecimal(0);
            int totalNumber = 0;

            List<InsuranceJobsDTO> insuranceJobsList = new ArrayList<>();

            //查询不同岗位，参保费用
            for(InsuranceJobs insuranceJobs : InsuranceJobsList){
                Jobs jobs = jobsService.get(insuranceJobs.getJobsId());

                InsuranceJobsDTO insuranceJobsDTO = new InsuranceJobsDTO();

                insuranceJobsDTO.setJobs(jobs);

                long memberCount = insurancePolicyMemberService.countByPolicyIdAndJobsId(policyId,insuranceJobs.getJobsId());

                insuranceJobsDTO.setCharges(insuranceJobs.getCharges());
                insuranceJobsDTO.setChargesY(insuranceJobs.getCharges().multiply(new BigDecimal(12)));
                insuranceJobsDTO.setNumber(Long.valueOf(memberCount).intValue());

                insuranceJobsDTO.setMoney(insuranceJobsDTO.getCharges()
                        .multiply(new BigDecimal(months))
                        .multiply(new BigDecimal(insuranceJobsDTO.getNumber())));

                totalAmount = totalAmount.add(insuranceJobsDTO.getMoney());
                totalNumber += memberCount;

                insuranceJobsList.add(insuranceJobsDTO);
            }

            acceptInsuranceDTO.setInsuranceJobsList(insuranceJobsList);

            //险种关联限额
            List<InsuranceDefinitionLimit> insuranceDefinitionLimits = insuranceDefinitionLimitService.findByDefinitionId(insuranceApplication.getDefinitionId());

            Map<String,Object> varsMap = new HashMap<>();
            varsMap.put("num",totalNumber);

            for(InsuranceDefinitionLimit  insuranceDefinitionLimit : insuranceDefinitionLimits){
                //限额这里可能是公式，需要带入人数
                String limit = VelocityHelper.format(insuranceDefinitionLimit.getLimit(),varsMap);
                insuranceDefinitionLimit.setLimit(limit);

                String unit = VelocityHelper.format(insuranceDefinitionLimit.getUnit(),varsMap);
                insuranceDefinitionLimit.setUnit(unit);
            }

            acceptInsuranceDTO.setInsuranceDefLimitList(insuranceDefinitionLimits);

            Integer status = 0;

            if (StringUtils.isNotEmpty(insuranceApplication.getStatus())) {
                status = Integer.valueOf(insuranceApplication.getStatus());
            }

            //已提交后则显示为数据库中保存金额
            if(status>=PolicyStatus.PendingTrial.getValue() && insuranceApplication.getInsuranceFee()!=null){
                acceptInsuranceDTO.setTotalAmount(insuranceApplication.getInsuranceFee());
            }
            else {
                acceptInsuranceDTO.setTotalAmount(totalAmount);
            }

            msgResult.setData(acceptInsuranceDTO);
            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="提交保单")
    @RequestMapping(value = "submit",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "applicationId",value = "投保单编号", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "policyId",value = "每月投保单编号", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<InsuranceApplication> submit(String applicationId,
                                                      String policyId,
                                                      HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);

            int policyStatus = Integer.valueOf(insuranceApplication.getStatus());

            if (policyStatus!=PolicyStatus.Draft.getValue()
             && policyStatus!=PolicyStatus.Back.getValue()){
                throw new Exception("当前投保单状态为" + PolicyStatus.valueOf(Integer.valueOf(policyStatus)) + ",不能保存!");
            }

            DateTime endTime = new DateTime(insuranceApplication.getEndTime());
            DateTime startTime = new DateTime(insuranceApplication.getStartTime());

            //投保总月份
            int months = Months.monthsBetween(startTime, endTime.plusDays(1)).getMonths();

            BigDecimal totalAmount = new BigDecimal(0);
            int totalNumber = 0;

            List<InsuranceJobs> InsuranceJobsList = insuranceJobsService.findByDefinitionId(insuranceApplication.getDefinitionId());

            //查询不同岗位，参保费用
            for(InsuranceJobs insuranceJobs : InsuranceJobsList){
                long memberCount = insurancePolicyMemberService.countByPolicyIdAndJobsId(policyId,insuranceJobs.getJobsId());

                BigDecimal amount = insuranceJobs.getCharges()
                                    .multiply(new BigDecimal(memberCount));

                totalAmount = totalAmount.add(amount);
                totalNumber += memberCount;
            }

            insuranceApplication.setInsuranceFee(totalAmount.multiply(new BigDecimal(months)));
            insuranceApplication.setInsuredNumber(totalNumber);
            insuranceApplication.setStatus(String.valueOf(PolicyStatus.PendingTrial.getValue()));
            insuranceApplication.setUpdateBy(principal.getName());
            insuranceApplication.setUpdateTime(new Date());

//            if(StringUtils.isEmpty(insuranceApplication.getPolicyNo())) {
//                DateTime createTime = new DateTime(insuranceApplication.getCreateTime());
//                insuranceApplication.setPolicyNo(createTime.toString("yyyyMMddHHmmssSSS") + "-01");
//            }

            insuranceApplicationService.update(insuranceApplication);

            //更新每月投保单
            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);
            PolicyStatus srcPolicyStatus = PolicyStatus.Draft;

            if(StringUtils.isNotEmpty(insurancePolicy.getStatus())) {
                srcPolicyStatus = PolicyStatus.valueOf(Integer.valueOf(insurancePolicy.getStatus()));
            }

            Company company = companyService.get(insuranceApplication.getCompanyId());

            insurancePolicy.setLatestApplicationId(applicationId);
            insurancePolicy.setCompanyId(insuranceApplication.getCompanyId());

            insurancePolicy.setDefinitionId(insuranceApplication.getDefinitionId());
            insurancePolicy.setInsuranceFee(totalAmount);
            insurancePolicy.setNumber(totalNumber);
            insurancePolicy.setNo(1); //首月
            insurancePolicy.setPolicyNo(insuranceApplication.getPolicyNo()); //首页投保单相同
            insurancePolicy.setEffectiveDate(startTime.toString("yyyyMM"));

            insurancePolicy.setStartTime(startTime.toDate());
            insurancePolicy.setEndTime(startTime.plusMonths(1).plusDays(-1).toDate());
            insurancePolicy.setStatus(String.valueOf(PolicyStatus.PendingTrial.getValue()));
            insurancePolicy.setPromoter(company.getPromoter()); //保存一份推广人，以免企业退广人发生变化

            insurancePolicy.setUpdateBy(principal.getName());
            insurancePolicy.setUpdateTime(new Date());

            insurancePolicyService.update(insurancePolicy);

            templateMessageService.sendTemplateMessage(policyId,srcPolicyStatus,PolicyStatus.PendingTrial);

            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="保存草稿")
    @RequestMapping(value = "saveDraft",method = RequestMethod.POST)
    public MessageResult<InsuranceApplicationDTO> saveDraft(InsuranceApplicationDTO insuranceApplicationDTO,
                                                        HttpServletRequest request
    ){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
//        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<InsuranceApplicationDTO> msgResult = new MessageResult<>();
        InsuranceApplication insuranceApplication = null;

        try {
            int affectCount = 0;

            if(StringUtils.isNotEmpty(insuranceApplicationDTO.getApplicationId())) {
                insuranceApplication = insuranceApplicationService.get(insuranceApplicationDTO.getApplicationId());
            }

            DateTime startTime = new DateTime(insuranceApplicationDTO.getStartTime());

            //加月份后减一天
            DateTime endTime = startTime.plusMonths(insuranceApplicationDTO.getMonthNumber()).plusDays(-1);

            if(insuranceApplication==null){
                insuranceApplication = new InsuranceApplication();
                PojoUtils.map(insuranceApplicationDTO, insuranceApplication);

                insuranceApplication.setId(insuranceApplicationDTO.getApplicationId());
//                insuranceApplication.setCompanyId(company.getId());
                insuranceApplication.setStatus(String.valueOf(PolicyStatus.Draft.getValue()));

                insuranceApplication.setEndTime(endTime.toDate());

                insuranceApplication.setDelFlag(false);
                insuranceApplication.setCreateBy(principal.getName());
                insuranceApplication.setCombinePolicyId(null);

                DateTime createTime = DateTime.now();
                insuranceApplication.setCreateTime(createTime.toDate());

                String policyNo = insuranceApplicationService.generatePolicyNo(createTime.toDate(),1);

                insuranceApplication.setPolicyNo(policyNo);

                if ("null".equals(insuranceApplication.getPolicyNo())){
                    insuranceApplication.setPolicyNo(null);
                }

                if ("null".equals(insuranceApplication.getCombinePolicyId())){
                    insuranceApplication.setCombinePolicyId(null);
                }

                affectCount = insuranceApplicationService.insert(insuranceApplication);
            }else {
                int policyStatus = Integer.valueOf(insuranceApplication.getStatus());

                if (policyStatus!=PolicyStatus.Draft.getValue()
                 && policyStatus!=PolicyStatus.Back.getValue()){
                   throw new Exception("当前投保单状态为" + PolicyStatus.valueOf(Integer.valueOf(policyStatus)) + ",不能保存!");
                }

                PojoUtils.map(insuranceApplicationDTO, insuranceApplication);

                insuranceApplication.setEndTime(endTime.toDate());
                insuranceApplication.setUpdateBy(principal.getName());
                insuranceApplication.setUpdateTime(new Date());

                //insuranceApplicationDTO中增加了policyNo，导致第一次新增后不刷新页面点击保存为"null"，同时影响了合并投保单
                if ("null".equals(insuranceApplication.getPolicyNo())){
                    insuranceApplication.setPolicyNo(null);
                }

                if ("null".equals(insuranceApplication.getCombinePolicyId())){
                    insuranceApplication.setCombinePolicyId(null);
                }

                affectCount = insuranceApplicationService.update(insuranceApplication);
            }

            if(affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceApplicationDTO);
            }else {
                msgResult.setResult(false);
            }
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="删除投保单")
    @PostMapping("delete")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<Integer> delete(@RequestParam(value="id",defaultValue="") String id,
                                         HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(id);
            insuranceApplication.setDelFlag(true);
            insuranceApplication.setUpdateBy(principal.getName());
            insuranceApplication.setUpdateTime(new Date());

            int affectCount = insuranceApplicationService.update(insuranceApplication);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @PostMapping(value="changeStatus")
    @ApiOperation(value = "修改投保单状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "applicationId",value = "投保单编号", required = true, paramType = "form"),
            @ApiImplicitParam(name = "status",value = "状态:草稿10/已退回15/待初审20/待复审30/待缴费40/待制单50/待出单60/已出单70/已过期100", required = true, paramType = "form"),
    })
    public MessageResult<String> changeStatus(String applicationId,Integer status,HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

        MessageResult<String> messageResult = new MessageResult<>();

        try {
            PolicyStatus destStatus = PolicyStatus.valueOf(status);

            if (destStatus==null){
                throw new Exception("状态:" + status + "不存在!");
            }

            //todo 修改投保单状态
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);
            InsurancePolicy insurancePolicy = applicationPolicyService.findFirstPolicyByApplicationId(applicationId);

            if(insurancePolicy!=null) {
                //记录状态变更
                InsurancePolicyRecord insurancePolicyRecord = new InsurancePolicyRecord();
                insurancePolicyRecord.setPolicyId(insurancePolicy.getId());
                insurancePolicyRecord.setStatus(status + "");
                insurancePolicyRecord.setProcessStatus("正常");
                insurancePolicyRecord.setOpinion("状态由" + insuranceApplication.getStatusN() + "变更为" + destStatus.getText());
                insurancePolicyRecord.setId(UUID.randomUUID().toString());
                insurancePolicyRecord.setDelFlag(false);
                insurancePolicyRecord.setCreateBy(principal.getName());
                insurancePolicyRecord.setCreateTime(new Date());
                insurancePolicyRecordService.insert(insurancePolicyRecord);
            }

            insuranceApplication.setStatus(status + "");
            insuranceApplication.setUpdateTime(new Date());

            insuranceApplicationService.update(insuranceApplication);

            messageResult.setResult(true);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @ResponseBody
    @ApiOperation(value="缴费接口")
    @ApiImplicitParam(name = "applicationId",value = "投保单编号",paramType ="form",required = true)
    @PostMapping(value="/paySubmit")
    public MessageResult paySubmit(HttpServletRequest request,String applicationId){
        MessageResult msgResult = new MessageResult<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);

            if(insuranceApplication==null){
                throw new Exception("投保单不存在!");
            }

            if(!String.valueOf(PolicyStatus.PendingPay.getValue()).equals(insuranceApplication.getStatus())){
                throw new Exception("投保单当前状态不是待缴费状态!");
            }

            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            Company company = null;

            if(companyUser!=null && StringUtils.isNotEmpty(companyUser.getCompanyId())) {
                company = companyService.get(companyUser.getCompanyId());
            }

            String serialNumber = (String)principal.getAttributes().get("serialNumber");

            TransactionRecord transactionRecord = new TransactionRecord();
            transactionRecord.setId(UUID.randomUUID().toString());
            transactionRecord.setApplicationId(applicationId);

            if(company!=null) {
                transactionRecord.setCompanyId(company.getId());
            }

            transactionRecord.setAmount(insuranceApplication.getInsuranceFee());
            transactionRecord.setPaymentStatus("0");
            transactionRecord.setOrderNo(DateTime.now().toString("yyyyMMddHHmmssSSS"));
            transactionRecord.setCreateBy(principal.getName());
            transactionRecord.setCreateTime(new Date());

            int affectCount = transactionRecordService.insert(transactionRecord);

            if(affectCount==0){
                throw new Exception("创建交易单失败!");
            }

            HashMap<String, Object> paramMap = new HashMap<String, Object>();
            paramMap.put("orderNo", transactionRecord.getOrderNo());
            paramMap.put("orderName", insuranceApplication.getDefinitionName() + "网上缴费");
            paramMap.put("freezeTime", transactionRecord.getCreateTime().getTime() / 1000);
            paramMap.put("unfreezeTime", transactionRecord.getCreateTime().getTime() / 1000);
            paramMap.put("orderAmount", transactionRecord.getAmount());
            paramMap.put("code", jpCloudConfig.getPlatformCode()); //平台号
            paramMap.put("buyerSerialNumber", serialNumber);//取登录用户的ID
            paramMap.put("sellerSerialNumber", jpCloudConfig.getSellerSerialNumber());
            paramMap.put("notifyBackUrl", jpCloudConfig.getNotifyBackUrl());
            paramMap.put("synchroBackUrl", jpCloudConfig.getSynchroBackUrl());
            String sign = SignUtil.createSign(paramMap, jpCloudConfig.getPlatformKey());

            paramMap.put("orderName", URLEncoder.encode(insuranceApplication.getDefinitionName() + "网上缴费","UTF-8"));
            paramMap.put("notifyBackUrl", URLEncoder.encode(jpCloudConfig.getNotifyBackUrl(),"UTF-8"));
            paramMap.put("synchroBackUrl", URLEncoder.encode(jpCloudConfig.getSynchroBackUrl(),"UTF-8"));

            paramMap.put("sign", sign);

            String param = "";

            for (Map.Entry<String, Object> map : paramMap.entrySet()) {
                param += map.getKey() + "=" + map.getValue() + "&";
            }

            if (StringUtils.isNotBlank(param)) {
                param = param.substring(0, param.length() - 1);
            }

            String returnUrl = jpCloudConfig.getPayUrl() + "?" + param;

            msgResult.setData(returnUrl);
            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="投保单状态分类数量")
    @RequestMapping(value = "statusCount",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "definitionId",value = "险种编号", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "startTime",value = "起始时间", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "endTime",value = "结束时间", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "status",value = "投保单状态", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<Map> statusCount(String definitionId,
                                          @DateTimeFormat(pattern = "yyyy-MM-dd") Date startTime,
                                          @DateTimeFormat(pattern = "yyyy-MM-dd") Date endTime,
                                          String status,
                                          HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

        CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
        Company company = null;

        if(companyUser!=null && StringUtils.isNotEmpty(companyUser.getCompanyId())) {
            company = companyService.get(companyUser.getCompanyId());
        }

        MessageResult<Map> msgResult = new MessageResult<>();
        Map<String, Object> dataMap = new HashMap<>();

        try {
            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            if (company!=null && StringUtils.isNotEmpty(company.getId())) {
                searchParams.put("companyId", company.getId());
            }
            else{
                searchParams.put("companyId", "");
            }

            if (StringUtils.isNotEmpty(definitionId)) {
                searchParams.put("definitionId", definitionId);
            }

            if (startTime != null) {
                searchParams.put("startTime", startTime);
            }

            if (endTime != null) {
                searchParams.put("endTime", endTime);
            }

            for(PolicyStatus policyStatus : PolicyStatus.values()){
                boolean processed = false;

                if (StringUtils.isEmpty(status)){
                    processed = true;
                }
                else if(status.equals(String.valueOf(policyStatus.getValue()))){
                    processed = true;
                }

                if (processed) {
                    searchParams.put("status", policyStatus.getValue() + "");
                    Page<InsuranceApplication> page = insuranceApplicationService.pageSearch(searchParams, 1, 1, sortList);

                    dataMap.put(policyStatus.getValue() + "", page.getTotal());
                }
            }

            msgResult.setData(dataMap);
            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ResponseBody
    @ApiOperation(value="快速续保")
    @ApiImplicitParam(name = "applicationId",value = "投保单编号",paramType ="form",required = true)
    @PostMapping(value="/renew")
    @Transactional
    public MessageResult<String> renew(String applicationId,HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            InsuranceApplication srcApplication = insuranceApplicationService.get(applicationId);
            InsuranceApplication destApplication = new InsuranceApplication();

            PojoUtils.map(srcApplication,destApplication);

            destApplication.setId(UUID.randomUUID().toString());
            destApplication.setStatus(PolicyStatus.Draft.getValue()  + "");

            destApplication.setCreateBy(principal.getName());
            destApplication.setCreateTime(new Date());

            String policyNo = insuranceApplicationService.generatePolicyNo(destApplication.getCreateTime(),1);
            destApplication.setPolicyNo(policyNo);

            DateTime srcStartTime = new DateTime(srcApplication.getStartTime());
            DateTime srcEndTime = new DateTime(srcApplication.getEndTime());
            int days = Days.daysBetween(srcStartTime,srcEndTime).getDays();

            //todo 修改投保时间
            DateTime destStartTime = srcEndTime.plusDays(1);
            DateTime destEndTime = destStartTime.plusDays(days);

            destApplication.setStartTime(destStartTime.toDate());
            destApplication.setEndTime(destEndTime.toDate());
            destApplication.setCombinePolicyId(null);
            destApplication.setFinishTime(null);
            destApplication.setUpdateBy(null);
            destApplication.setUpdateTime(null);

            insuranceApplicationService.insert(destApplication);

            //todo 新增一份每月投保单
            InsurancePolicy destPolicy = new InsurancePolicy();
            destPolicy.setId(UUID.randomUUID().toString());
            destPolicy.setLatestApplicationId(destApplication.getId());
            destPolicy.setPolicyNo(destApplication.getPolicyNo());

            destPolicy.setDelFlag(false);
            destPolicy.setCreateBy(principal.getName());
            destPolicy.setCreateTime(new Date());
            destPolicy.setNo(1);

            int affectCount = insurancePolicyService.insert(destPolicy);

            InsurancePolicy lastPolicy = applicationPolicyService.findLastPolicyByApplicationId(srcApplication.getId());

            if (affectCount > 0) {
                //todo 将投保人导入
                List<InsurancePolicyMember> memberList = insurancePolicyMemberService.findByPolicyId(lastPolicy.getId());

                for (InsurancePolicyMember srcMember : memberList) {
                    InsurancePolicyMember destMember = new InsurancePolicyMember();

                    destMember.setId(UUID.randomUUID().toString());
                    destMember.setPolicyId(destPolicy.getId());
                    destMember.setMemberId(srcMember.getMemberId());
                    destMember.setDelFlag(false);
                    destMember.setCreateBy(principal.getName());
                    destMember.setCreateTime(new Date());
                    destMember.setStatus(srcMember.getStatus());

                    insurancePolicyMemberService.insert(destMember);
                }

                //todo 关联投保申请单月与每月投保单
                ApplicationPolicy applicationPolicy = new ApplicationPolicy();
                applicationPolicy.setId(UUID.randomUUID().toString());

                applicationPolicy.setApplicationId(destApplication.getId());
                applicationPolicy.setPolicyId(destPolicy.getId());

                applicationPolicyService.insert(applicationPolicy);
            }

            msgResult.setData(destApplication.getId());
            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
