package com.jpsoft.weixin.controller;

import cn.hutool.json.JSONObject;
import com.jpsoft.weixin.callback.EventCallback;
import com.jpsoft.weixin.callback.PromotionCallback;
import com.jpsoft.weixin.config.PICCAdminScanConfig;
import com.jpsoft.weixin.config.PICCEntScanConfig;
import com.jpsoft.weixin.config.WeixinConfig;
import com.jpsoft.weixin.entity.ReplyMessage;
import com.jpsoft.weixin.service.ReplyMessageService;
import com.jpsoft.weixin.utils.VelocityHelper;
import com.jpsoft.weixin.utils.WeixinUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Api(description = "微信中控")
@Controller
@Slf4j
public class WeixinController {
    @Autowired
    private WeixinConfig weixinConfig;

    @Autowired
    private PICCEntScanConfig piccEntScanConfig;

    @Autowired
    private PICCAdminScanConfig piccAdminScanConfig;

    @Autowired
    private ValueOperations<String,Object> valueOperations;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private ReplyMessageService replyMessageService;

    @ApiOperation(value="验证服务器地址的有效性")
    @GetMapping("/")
    @ResponseBody
    public String index(@RequestParam(defaultValue = "") String signature,
                        @RequestParam(defaultValue = "") String timestamp,
                        @RequestParam(defaultValue = "") String nonce,
                        @RequestParam(defaultValue = "") String echostr){
        log.warn("signature=" + signature);
        log.warn("timestamp=" + timestamp);
        log.warn("nonce=" + nonce);
        log.warn("echostr=" + echostr);

        if (WeixinUtil.checkSignature(weixinConfig.getToken(),signature,timestamp,nonce)){
            return echostr;
        }
        else{
            return "index";
        }
    }

    @ApiOperation(value = "基础支持中获取普通access_token")
    @GetMapping("/token")
    @ResponseBody
    public JSONObject getToken(
            @RequestParam(defaultValue = "client_credential") String grant_type,
            @RequestParam(defaultValue = "") String appid,
            @RequestParam(defaultValue = "") String secret){
        String suffix = "_common_token";
        JSONObject json = (JSONObject)valueOperations.get(appid + suffix);

        if(json==null) {
            StringBuilder urlBuilder = new StringBuilder();

            urlBuilder.append(weixinConfig.getTokenUrl());

            urlBuilder.append("?").append("grant_type=").append(grant_type);
            urlBuilder.append("&").append("appid=").append(appid);
            urlBuilder.append("&").append("secret=").append(secret);

            try {
                HttpGet httpGet = new HttpGet(urlBuilder.toString());

                CloseableHttpClient httpClient = HttpClientBuilder.create().build();

                CloseableHttpResponse response = httpClient.execute(httpGet);

                HttpEntity entity = response.getEntity();

                String content = EntityUtils.toString(entity);

                json = new JSONObject(content);

                if(json.containsKey("expires_in")){
                    Long expiresSeconds = json.getLong("expires_in");
                    valueOperations.set(appid + suffix,json,expiresSeconds - 30, TimeUnit.SECONDS);
                }
            } catch (Exception ex) {
                log.error(ex.getMessage(), ex);
            }
        }

        return json;
    }

    @ApiOperation(value = "刷新基础支持token")
    @GetMapping("/refreshToken")
    @ResponseBody
    public Boolean refreshToken(@RequestParam(defaultValue = "") String appid){
        String suffix = "_common_token";

        return redisTemplate.delete(appid + suffix);
    }

    @ApiOperation(value = "处理消息事件")
    @PostMapping("/")
    public void processEvent(HttpServletRequest request, HttpServletResponse response){
        try{
            log.warn("收到post请求");

            JSONObject requestJson = null;

            if("json".equals(weixinConfig.getContentType())){
                String jsonStr = new BufferedReader(new InputStreamReader(request.getInputStream()))
                        .lines().parallel().collect(Collectors.joining(System.lineSeparator()));

                requestJson = new JSONObject(jsonStr);
            }
            else {
                requestJson = WeixinUtil.xmlToJson(request.getInputStream());
            }

            //{Ticket=gQEH8DwAAAAAAAAAAS5odHRwOi8vd2VpeGluLnFxLmNvbS9xLzAyckZDekV1RHg5SFAxczNwYjF1MWIAAgTX10teAwQsAQAA, CreateTime=1582028776, EventKey=1457, Event=SCAN, ToUserName=gh_b6e865f48dea, FromUserName=op6Frsy4xuXYGqjaJ12Xv5Q-9NtU, MsgType=event}
            log.warn(requestJson.toString());

            response.setContentType("UTF-8");
            response.setContentType("text/html; charset=UTF-8");
            response.setCharacterEncoding("UTF-8");

            String event = requestJson.getStr("Event");
            String eventKey = requestJson.getStr("EventKey");

            /*
            正常扫码是："EventKey":"7000,2142006"；
            当未关注时，扫码是"EventKey":"qrscene_7000,2142006"
            */
            if(StringUtils.isNotEmpty(eventKey) && eventKey.indexOf("_")!=-1){
                String[] arr = eventKey.split("_");

                if(arr.length>0){
                    eventKey = arr[1];
                }
            }

            //微信公众号编号
            String wechatId = requestJson.getStr("ToUserName");
            //扫码用户openID
            String openId = requestJson.getStr("FromUserName");

//            List<EventCallback> callbackList = registerCallbackList();

            boolean processed = false;

            if (StringUtils.isNotEmpty(eventKey) && eventKey.indexOf(",")!=-1) {
                String[] keys = eventKey.split(",");
                String eventCode = keys[0];

                ReplyMessage replyMessage = replyMessageService.findByEvent(wechatId, eventCode);

                if (replyMessage != null) {
                    EventCallback callback = new EventCallback(replyMessage.getEvent(), replyMessage.getCallbackUrl());

                    callback.process(wechatId, openId, eventKey);

                    WeixinUtil.replyTextMessage(response, wechatId, openId, callback.getMessage());

                    processed = true;
                }
            }
                //开发者在5秒内未回复任何内容
//                for (EventCallback callback : callbackList) {
//                    if (StringUtils.isNotEmpty(eventKey) && eventKey.startsWith(callback.getCode())) {
//                        callback.process(wechatId, openId, eventKey);
//
//                        WeixinUtil.replyTextMessage(response, wechatId, openId, callback.getMessage());
//
//                        processed = true;
//                        break;
//                    }
//                }
//            }

            if(!processed) {
                String responseText = "success";

                if("subscribe".equals(event)){
                    ReplyMessage replyMessage = replyMessageService.findByEvent(wechatId,event);

                    if(replyMessage!=null) {
                        //todo 自动回复新增了投保链接需要获取当前用户的openId
                        StringBuilder urlBuilder = new StringBuilder();

                        urlBuilder.append("openId=" + openId);
                        urlBuilder.append("&promoter=0"); //自动回复带的推广人为0，这种情况就不保存到数据库

                        Map<String, Object> vars = new HashMap<>();
                        vars.put("params", urlBuilder.toString());

                        String message = VelocityHelper.format(replyMessage.getMessage(), vars);
                        WeixinUtil.replyTextMessage(response, wechatId, openId, message);
                    }
                    else{
                        log.warn(openId + ",事件:" + event + "未设置自动回复!");
                        response.getWriter().print(responseText);
                    }
                }
                else {
                    response.getWriter().print(responseText);
                }
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            try {
                response.getWriter().print("fail");
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private List<EventCallback> registerCallbackList() {
        List<EventCallback> list = new ArrayList<>();

        list.add(new EventCallback(piccEntScanConfig.getLoginCode(),piccEntScanConfig.getLoginCallbackUrl()));
        list.add(new EventCallback(piccEntScanConfig.getBindingCode(),piccEntScanConfig.getBindingCallbackUrl()));
        list.add(new EventCallback(piccAdminScanConfig.getLoginCode(),piccAdminScanConfig.getLoginCallbackUrl()));
        list.add(new EventCallback(piccAdminScanConfig.getBindingCode(),piccAdminScanConfig.getBindingCallbackUrl()));
        list.add(new EventCallback(piccAdminScanConfig.getPromotionCode(),piccAdminScanConfig.getPromotionCallbackUrl()));

//        PromotionCallback promotionCallback = new PromotionCallback(piccAdminScanConfig.getPromotionCode(),piccAdminScanConfig.getPromotionCallbackUrl());
//        promotionCallback.setReplyMessageService(replyMessageService);



        return list;
    }

    @ResponseBody
    @ApiOperation(value = "创建二维码")
    @PostMapping("/qrcode/create")
    @ApiImplicitParams({
            @ApiImplicitParam(name="access_token", paramType="query", required=true, value="基础支持access_token"),
            @ApiImplicitParam(name="expire_seconds", paramType="query", required=true, value="该二维码有效时间，以秒为单位。 最大不超过2592000（即30天），此字段如果不填，则默认有效期为30秒。"),
            @ApiImplicitParam(name="action_name", paramType="query", required=true, value="二维码类型，QR_SCENE为临时的整型参数值，QR_STR_SCENE为临时的字符串参数值，QR_LIMIT_SCENE为永久的整型参数值，QR_LIMIT_STR_SCENE为永久的字符串参数值"),
            @ApiImplicitParam(name="scene_id", paramType="query", value="场景值ID，临时二维码时为32位非0整型，永久二维码时最大值为100000（目前参数只支持1--100000）"),
            @ApiImplicitParam(name="scene_str", paramType="query", value="场景值ID（字符串形式的ID），字符串类型，长度限制为1到64")
    })
    public JSONObject createQrcode(@RequestParam(name = "access_token") String accessToken,
                               @RequestParam(name = "expire_seconds",defaultValue = "300") long expireSeconds,
                               @RequestParam(name = "action_name",defaultValue = "QR_STR_SCENE") String actionName,
                               @RequestParam(name = "scene_id",required = false) String sceneId,
                               @RequestParam(name = "scene_str",required = false) String sceneStr){
        JSONObject resultObj = new JSONObject();

        try {
            StringBuilder urlBuilder = new StringBuilder();

            urlBuilder.append(weixinConfig.getCreateQrCodeUrl())
                    .append("?access_token=")
                    .append(URLEncoder.encode(accessToken,"UTF-8"));

            JSONObject jsonObject = new JSONObject();
            jsonObject.put("expire_seconds", expireSeconds);
            jsonObject.put("action_name", actionName);

            JSONObject actionInfo = new JSONObject();
            JSONObject scene = new JSONObject();

            StringBuilder keyBuilder = new StringBuilder();

            keyBuilder.append(accessToken);

            if(StringUtils.isNotEmpty(sceneId)) {
                scene.put("scene_id", sceneId);
                keyBuilder.append(sceneId);
            }

            if(StringUtils.isNotEmpty(sceneStr)) {
                scene.put("scene_str", sceneStr);
                keyBuilder.append(sceneStr);
            }

            String qrcodeUrl = (String)valueOperations.get("QRCODE_" + keyBuilder.toString());

            if (StringUtils.isNotEmpty(qrcodeUrl)){
                resultObj.put("data",qrcodeUrl);
                resultObj.put("result", true);
            }
            else{
                actionInfo.put("scene",scene);

                jsonObject.put("action_info", actionInfo);

                StringEntity entity = new StringEntity(jsonObject.toString(), "utf-8");

                entity.setContentType("application/json");//发送json数据需要设置contentType

                HttpPost httpPost = new HttpPost(urlBuilder.toString());
                CloseableHttpClient httpClient = HttpClientBuilder.create().build();

                httpPost.setEntity(entity);

                HttpResponse res = httpClient.execute(httpPost);

                if (res.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
                    String content = EntityUtils.toString(res.getEntity());// 返回json格式：
                    JSONObject ret = new JSONObject(content);

                    if (ret.containsKey("errcode")) {
                        resultObj.put("result", false);
                        resultObj.put("message", ret.getStr("errmsg"));
                        resultObj.put("code", ret.getInt("errcode"));
                    } else {
                        String ticket = ret.getStr("ticket");
                        long expire_seconds = ret.getLong("expire_seconds");

                        qrcodeUrl = weixinConfig.getShowQrCodeUrl() + "?ticket=" + URLEncoder.encode(ticket, "UTF-8");

                        valueOperations.set("QRCODE_" + keyBuilder, qrcodeUrl, expire_seconds - 5, TimeUnit.SECONDS);

                        resultObj.put("data", qrcodeUrl);
                        resultObj.put("result", true);
                    }
                }
                else{
                    resultObj.put("result", false);
                    resultObj.put("message", "weixin服务器未正常返回!");
                }
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            resultObj.put("message",ex.getMessage());
            resultObj.put("result",false);
        }

        return resultObj;
    }
}
